function [all_estimates bestval best_exitflag allMaxVals] = ...
    estimate_model(filename, printscreen, writetemp, ...
                   display_simplex, display_deriv, restart_index)
%
% ESTIMATE_MODEL : reads the control file 'filename' and estimates the
% term structure model.  Some intermediate results are printed to
% the screen if 'printscreen' == 1.  
%
% Greg Duffee
% September 2009
%

% assume this is not a restart if there is no restart index. 
if (nargin<6); restart_index = 1; end;

tolerance = 1e-12; 

[inputfile outputfile] = read_control(filename);

[yielddatafile numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro numUsedMacro usedMacroIdx ...
 numfactors firstobs lastobs ...
 use_ols_start olsStartIdx ...
 numloops numSmallLoops numsimplex numderiv ...
 mu_names mu_flag mu_initialval mu_rand_stddev ...
 k_names k_flag k_initialval k_rand_stddev ...
 S_names S_flag S_initialval S_rand_stddev ...
 A_name A_flag A_initialval A_rand_stddev ...
 B_names B_flag B_initialval B_rand_stddev ...
 errorSD_name errorSD_flag errorSD_initialval errorSD_rand_stddev ...
] = read_input(inputfile);

%
% don't bother writing temporary results if a single set of
% starting values is used.
%

writetemp_copy = writetemp; 
if (numloops < 2)
  writetemp_copy = 0;
end

observedData = ...
    read_data(yielddatafile, macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs);

[freeparams bigparamvec free_index] = setup_initparams( ...
    mu_initialval, k_initialval, S_initialval, ...
    A_initialval, B_initialval, ...
    errorSD_initialval, mu_flag, k_flag, S_flag, ...
    A_flag, B_flag, errorSD_flag, use_ols_start, olsStartIdx, ...
    observedData);


stddevs = pack_allparams( ...
    mu_rand_stddev, k_rand_stddev, ...
    S_rand_stddev, A_rand_stddev, ...
    B_rand_stddev, errorSD_rand_stddev, ...
    mu_flag, k_flag, S_flag, A_flag, ...   
    B_flag, errorSD_flag);

if (printscreen == 1) 

  fprintf(1, ['\n\nMaximum likelihood estimation of a ', ...
              'factor model\n']);
  fprintf(1,     ['-----------------------------------', ...
                  '-------------\n\n']);
  fprintf(1, 'Input file is  %s.\n', inputfile);
  fprintf(1, 'Output file is %s.\n', outputfile);
  fprintf(1, 'Data files are:\n');
  fprintf(1, '%s\n', yielddatafile);
  fprintf(1, '%s\n\n', macrodatafile);
  if (numloops == 1) 
    fprintf(1, ['A single set of starting parameters is used to ', ...
                'begin the optimization.\n']);
  end
  if (numloops > 1) 
    fprintf(1, ['%d sets of starting parameters are used to ', ...
                'begin the optimization.\n'], numloops);
    fprintf(1, 'All but the first are randomly generated.\n'); 
  end
  if (writetemp_copy == 1) 
    fprintf(1, ['Temporary results are written to tempparams.out ', ...
                'and tempindex.out.\n']); 
  else
    fprintf(1, 'Temporary results are not written anywhere.\n'); 
  end
  fprintf(1, '\n'); 
end

numObserved = numUsedBonds + numUsedMacro; 

[mu k_vec Sigma_vec A B_vec errorSDs] = ...
    unpack_allparams(numfactors, numObserved, bigparamvec);

[Kmat Sigma B H] = ...
    build_matrices(k_vec, Sigma_vec, B_vec, errorSDs); 

[funval loglikes filtered_values] = ...
    kalman_only_loglike(freeparams, bigparamvec, free_index, ...
                        observedData, numfactors, tolerance); 


[free_estimates all_estimates bestval best_exitflag, ...
         allMaxVals] = ...
    solve_ml(freeparams, bigparamvec, free_index, ...
             observedData, numfactors, tolerance, ... 
             numloops, numSmallLoops, ...
             numsimplex, numderiv, stddevs, printscreen, ...
             writetemp_copy, display_simplex, display_deriv, ...
             restart_index);

[funval loglikes filtered_values] = ...
    kalman_only_loglike(free_estimates, all_estimates, free_index, ...
                        observedData, numfactors, tolerance); 

[mu k_vec Sigma_vec A B_vec errorSDs] = ...
    unpack_allparams(numfactors, numObserved, all_estimates);

% Apply some normalizations.

Sigma = zeros(numfactors, numfactors); 
index = 0; 
for i = 1:numfactors
  for j = 1:i
    index = index + 1; 
    Sigma(i,j) = Sigma_vec(index);
  end
end
for i = 1:numfactors
  Sigma(:,i) = sign(Sigma(i,i))*Sigma(:,i);
end
index = 0; 
for i = 1:numfactors
  for j = 1:i
    index = index + 1; 
    Sigma_vec(index) = Sigma(i,j);
  end
end

delta = 1e-8; 
[score outerProd scoreMat] = ...
    kalman_derivs(free_estimates, all_estimates, free_index, ...
                  observedData, numfactors, tolerance, ...
                  delta);


write_output(inputfile, outputfile, mu, k_vec, Sigma_vec, ...
                      A, B_vec, errorSDs, bestval, best_exitflag);

return

