function [filteredContempStates smoothedStates ...
          filteredContempCov smoothedCov] = ...
    kalman_smoothing_recursion(yields, mu, Kmat, Sigma, H, A, B)
% KALMAN_SMOOTHING_RECURSION constructs the entire Kalman
% recursion, both filtering and smoothing.  

%
% Greg Duffee
% February 2008, revised June 2010
%

[T numbonds] = size(yields); 
n = size(Kmat, 1); 

uncon_mean = inv(eye(n) - Kmat) * mu;
Q = Sigma*Sigma'; 
[uncon_vcv valid] = uncon_vcv_general(Kmat, Sigma);

if (valid == 0) 
  filteredContempStates = []; 
  filteredContempCov = []; 
  smoothedStates = []; 
  smoothedCov = []; 
else
%
% loop through all observations.
%
  errors = zeros(T, numbonds); 
  loglikes = zeros(T, 1); 
  filteredContempStates = zeros(T, n); 
  filteredCondStates = zeros(T, n); 
  smoothedStates = zeros(T, n); 
  filteredContempCov = zeros(T, (n*(n+1)/2));
  filteredCondCov = zeros(T, (n*(n+1)/2)); 
  smoothedCov = zeros(T, (n*(n+1)/2)); 
  
  Q = Sigma*Sigma'; 
  cond_expect = uncon_mean;
  cond_var = uncon_vcv; 

  % first run recursion to end of sample.  
  
  for k = 1:T
    observed_data = yields(k,:)';
    [error loglike contemp_expect contemp_var ...
     forecast_expect forecast_var] = ...
        single_recursion_noderiv(observed_data, cond_expect, cond_var, ...
                                 mu, A, B, H, Kmat, Q);
    filteredContempStates(k,:) = contemp_expect';
    filteredCondStates(k,:) = forecast_expect'; 
    filteredCondCov(k,:) = vech(forecast_var)'; 
    filteredContempCov(k,:) = vech(contemp_var)';
    cond_expect = forecast_expect;
    cond_var = forecast_var; 
  end
  
  % now run recursion backwards to smooth.  See section 3.6.2 of
  % Harvey.  For observation T, smoothed and filtered observations
  % are the same.  
  
  smoothedStates(T,:) = filteredContempStates(T,:);
  smoothedCov(T,:) = filteredContempCov(T,:); 
  next_smooth_state = contemp_expect;  % final filtered value
  next_smooth_var = contemp_var; % final filtered value
  
  for k = ((T-1):-1:1)
    contemp_state = filteredContempStates(k,:)'; 
    contemp_var = vech_inv(filteredContempCov(k,:)');
    cond_var = vech_inv(filteredCondCov(k,:)');
    Pstar = contemp_var*Kmat'*inv(cond_var);
    smooth_state = contemp_state + ...
        Pstar*(next_smooth_state - mu - Kmat*contemp_state);
    smooth_var = contemp_var + ...
        Pstar*(next_smooth_var - cond_var)*Pstar';
    smoothedStates(k,:) = smooth_state'; 
    smoothedCov(k,:) = vech(smooth_var)'; 
    next_smooth_state = smooth_state;
    next_smooth_var = smooth_var; 
  end

end
return
