function [OmegaK,varargout]=RS(K,A11,B11,B12,C,H,F)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Forward Recursive calculation OF  Omega, Gamma, stability conditions
% Structurel Model :  B11*x(t)=A11*E[x(t+K)|I(t)]+B12*x(t-1)+C*z(t)+e(t)
% Reduced    Model :      x(t)=MK*E[x(t+K)|I(t)]+Omega(K)x(t-1)+Gammaz(K)z(t)+Gamma(K)epsilon(t)
% Input :   1. K : number of iteration
%           2. A11,B11,B12 : n by n coefficient matrices
%           3. C : coefficient to z(t),C is n by m  (Optional)
%           4. H : z(t)=Hz(t-1)+u(t),  H is m by m  (Optional)
%           5, F : e(t)=Fe(t-1)+w(t),  F is n by n  (Optional)
% Output :  x(t)=MK*E[x(t+K)|I(t)]+Omega(K)x(t-1)+Gammaz(K)z(t)+Gamma(K)epsilon(t) 
%       [OmegaK, GeigK,GammaK,GammazK,MK,IAB,SK]
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%[0] size of the matrix
        n=cols(B11); 
        if      nargin==4, C=zeros(n,1); H=0; F=zeros(n,n);    %elseif  nargin==6, F=zeros(n,n);  
        end
        m=cols(C); 
%[1] Initial values of matrices                    
        phi1=inv(B11)*A11; psi1=inv(B11)*B12; p1=inv(B11)*C*H; q1=inv(B11)*F;
        
%[2] Define Sequences of Matrices
        Phi=[phi1;zeros(n*(K-1),n)];
        Psi=[psi1;zeros(n*(K-1),n)];
        P=  [p1;zeros(n*(K-1),m)];
        Q=  [q1;zeros(n*(K-1),n)];
        
        IAB=[];
        for i=2:K
            IAB0=inv(eye(n)-psi1*Phi(n*(i-2)+1:n*(i-1),:));      
            Phi(n*(i-1)+1:n*i,:)=IAB0*phi1;
            Psi(n*(i-1)+1:n*i,:)=IAB0*psi1*Psi(n*(i-2)+1:n*(i-1),:);
            P(n*(i-1)+1:n*i,:)  =IAB0*(psi1*P(n*(i-2)+1:n*(i-1),:)+p1*H^(i-1));
            Q(n*(i-1)+1:n*i,:)  =IAB0*(psi1*Q(n*(i-2)+1:n*(i-1),:)+q1*F^(i-1));  
            IAB=[IAB;IAB0];
        end;          

%[3] Construct Solution Sequences
        Phi0=[eye(n);phi1;zeros(n*(K-1),n)];
            for j=2:K
                Phi0(n*j+1:n*(j+1),:)=Phi0(n*(j-1)+1:n*j,:)*Phi(n*(j-1)+1:n*j,:);
            end;  % Phi0=[I phi1 phi1phi2 phi1phi2phi3.... phi1phi2phi3...phi(K)] (K+1)by 3

        SK0=[]; G1K0=[]; G2K0=[];
            for i=1:K
                SK0 = [SK0; vec(Phi0(n*(i-1)+1:n*i,:)*Psi(n*(i-1)+1:n*i,:))'];
                G1K0=[G1K0; vec(Phi0(n*(i-1)+1:n*i,:)*P(n*(i-1)+1:n*i,:))'];
                G2K0=[G2K0; vec(Phi0(n*(i-1)+1:n*i,:)*Q(n*(i-1)+1:n*i,:))'];
            end
            
%[4] Define Soution            
        SK =reshape( sum(SK0)',n,n);
        G1K=reshape(sum(G1K0)',n,m);
        G2K=reshape(sum(G2K0)',n,n);
        
        MK=inv(B11-A11*SK)*A11*Phi0(n*K+1:n*(K+1),:);
        OmegaK=inv(B11-A11*SK)*B12;
        GammazK=inv(B11-A11*SK)*(C+A11*G1K);
        GammaK=inv(B11-A11*SK)*(eye(n)+A11*G2K);

        
%[5] eigenvalues of Omega
        GeigK=eig(OmegaK);

% Output
varargout(1)={GeigK};
varargout(2)={GammaK};
varargout(3)={GammazK};
varargout(4)={MK};
varargout(5)={IAB};
varargout(6)={SK};
