function [all_estimates bestval best_exitflag allMaxVals] = ...
    estimate_model(filename, printscreen, writetemp, ...
                   display_simplex, display_deriv, restart_index)
%
% ESTIMATE_MODEL : reads the control file 'filename' and estimates the
% term structure model.  Some intermediate results are printed to
% the screen if 'printscreen' == 1.  
%
% Greg Duffee
% September 2009
%

% assume this is not a restart if there is no restart index. 
if (nargin<6); restart_index = 1; end;

[inputfile outputfile] = read_control(filename);

[yielddatafile numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro usedMacroIdx ...
 firstobs lastobs  ...
 numloops numSmallLoops numsimplex numderiv ...
 means_names means_flag means_initialval means_rand_stddev ...
 bcm_names bcm_flag bcm_initialval bcm_rand_stddev ...
 stddev_names stddev_flag stddev_initialval stddev_rand_stddev ...
 A_names A_flag A_initialval A_rand_stddev ...
 B_names B_flag B_initialval B_rand_stddev ...
 errorSD_names errorSD_flag errorSD_initialval errorSD_rand_stddev ...
] = read_input(inputfile);

%
% don't bother writing temporary results if a single set of
% starting values is used.
%

writetemp_copy = writetemp; 
if (numloops < 2)
  writetemp_copy = 0;
end

observedData = ...
    read_data(yielddatafile, ...
              macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs); 

[freeparams bigparamvec free_index] = setup_initparams( ...
    means_initialval, bcm_initialval, stddev_initialval, ...
    A_initialval, B_initialval, errorSD_initialval, ...
    means_flag, bcm_flag, stddev_flag, ...
    A_flag, B_flag, errorSD_flag);

stddevs = pack_allparams( ...
    means_rand_stddev, bcm_rand_stddev, ...
    stddev_rand_stddev, A_rand_stddev, ...
    B_rand_stddev, errorSD_rand_stddev, ...
    means_flag, bcm_flag, stddev_flag, A_flag, ...   
    B_flag, errorSD_flag);

if (printscreen == 1) 

  fprintf(1, ['\n\nMaximum likelihood estimation of a ', ...
              'factor model\n']);
  fprintf(1,     ['-----------------------------------', ...
                  '-------------\n\n']);
  fprintf(1, 'Input file is  %s.\n', inputfile);
  fprintf(1, 'Output file is %s.\n', outputfile);
  fprintf(1, 'Data files are:\n');
  fprintf(1, '%s\n', yielddatafile);
  fprintf(1, '%s\n\n', macrodatafile);
  if (numloops == 1) 
    fprintf(1, ['A single set of starting parameters is used to ', ...
                'begin the optimization.\n']);
  end
  if (numloops > 1) 
    fprintf(1, ['%d sets of starting parameters are used to ', ...
                'begin the optimization.\n'], numloops);
    fprintf(1, 'All but the first are randomly generated.\n'); 
  end
  if (writetemp_copy == 1) 
    fprintf(1, ['Temporary results are written to tempparams.out ', ...
                'and tempindex.out.\n']); 
  else
    fprintf(1, 'Temporary results are not written anywhere.\n'); 
  end
  fprintf(1, '\n'); 
end

[funval loglikes filtered_values] = ...
    kalman_only_loglike(freeparams, bigparamvec, free_index, ...
                        observedData);

[free_estimates all_estimates bestval best_exitflag ...
          best_grad allMaxVals] = ...
    solve_ml(freeparams, bigparamvec, free_index, ...
             observedData, ...
             numloops, numSmallLoops, ...
             numsimplex, numderiv, stddevs, printscreen, ...
             writetemp_copy, display_simplex, display_deriv, ...
             restart_index);

[funval loglikes filtered_values] = ...
    kalman_only_loglike(free_estimates, all_estimates, free_index, ...
                        observedData);

[means, bcm, stddev, Avec, Bvec, errorSD] = ...
    unpack_allparams(all_estimates, numUsedBonds); 

RE_output = solveRE(bcm, stddev); 

write_output(inputfile, outputfile, means, bcm, stddev, Avec, ...
                      Bvec, errorSD, bestval, best_exitflag);

return

