function [errors loglikes filtered_values cond_vars contemp_vars ...
         normSqrErrors eigFs] = ...
    kalman_recursion_noderiv(allObservations, mu, Kmat, Sigma, H, A, B, ...
                             tolerance)
% KALMAN_RECURSION_NODERIV constructs the entire Kalman recursion.

%
% Greg Duffee
% June 2011
%


physical_eigs = eig(Kmat); 
if (max(abs(physical_eigs)) > 1.0) 
%  fprintf(1, 'Nonstationary\n'); 
  loglikes = -1d8 - 1d9*(max(abs(physical_eigs)) - 1.0);
  errors = [];
  filtered_values = []; 
  cond_vars = []; 
  contemp_vars = []; 
  normSqrErrors = [];
  eigFs = []; 
else

  [T dataDim] = size(allObservations); 
  n = size(Kmat, 1); 

  uncon_mean = inv(eye(n) - Kmat) * mu;
  Q = Sigma*Sigma'; 
  [uncon_vcv_mat valid] = uncon_vcv_general(Kmat, Sigma);

  
  if (valid == 0) 
%    fprintf(1, 'Invalid unconditional variance\n'); 
    loglikes = -9d10; 
    errors = [];
    filtered_values = []; 
    cond_vars = []; 
    contemp_vars = []; 
    normSqrErrors = [];
    eigFs = []; 
  else
    %
    % loop through all observations.
    %
    errors = zeros(T, dataDim); 
    loglikes = zeros(T, 1); 
    filtered_values = zeros(T, n); 
    cond_vars = zeros(T, (n*(n+1)/2));
    contemp_vars = zeros(T, (n*(n+1)/2)); 
    normSqrErrors = zeros(T, 1);
    eigFs = zeros(T, dataDim); 
    
    cond_expect = uncon_mean;
    cond_var = uncon_vcv_mat; 
  
    %
    % Do each period of the recursion separately until the conditional
    % variances are sufficiently close.  Then invert the matrices one
    % last time, and run through the remaining observations.
    
    index = 1;
    old_cond_var = vech(cond_var); 
    not_same = 1;
    bailout = 0; 
    while ((index <= T) & (not_same == 1) & (bailout == 0) )
      y = allObservations(index,:)';
      [error loglike contemp_expect contemp_var ...
       forecast_expect forecast_var thisNormSqrError thisEigF] = ...
          single_recursion_noderiv(y, cond_expect, cond_var, ...
                                   mu, A, B, H, Kmat, Q);
      loglikes(index) = loglike;
      errors(index,:) = error';
      filtered_values(index,:) = contemp_expect';
      cond_expect = forecast_expect;
      cond_var = forecast_var; 
      cond_vars(index,:) = vech(cond_var)';
      contemp_vars(index,:) = vech(contemp_var)';
      normSqrErrors(index) = thisNormSqrError;
      eigFs(index,:) = thisEigF'; 
      if (thisNormSqrError > 0)
        index = index + 1; 
        if (sum(abs(vech(cond_var) - old_cond_var)) < tolerance)
          not_same = 0; 
        else
          old_cond_var = vech(cond_var);
        end
      else
        bailout = 1;
      end
      
    end
      
    %
    % loop through all other observations, using the last conditional
    % variance from above. 
    %
    if ((T >= index)  & (bailout == 0)) 
      F = B*cond_var*B' + H; 
      endEigF = eig(F); 
      detval = log(abs(det(F))); 
      [invF okinv] = invpd(F); 
      if (okinv == 0) 
        loglikes(index + 1) = -1e10; 
        thisNormSqrError = []; 
      else
        for k = index:T
          y = allObservations(k,:)';
          predicted_y = A + B*cond_expect;
          error = y - predicted_y; 
          thisNormSqrError = error' * invF * error; 
          loglike = -0.5*(detval + thisNormSqrError);
          info_in_errors = invF * error; 
          contemp_expect = cond_expect + cond_var * B' * info_in_errors;
          cond_expect = mu + Kmat*contemp_expect;
          loglikes(k) = loglike; 
          errors(k,:) = error'; 
          filtered_values(k,:) = contemp_expect'; 
          cond_vars(k,:) = vech(cond_var)';
          contemp_vars(k,:) = vech(contemp_var)';
          normSqrErrors(k) = thisNormSqrError;
          eigFs(k,:) = thisEigF'; 
        end
      end
    end
  end
end

return
