function [free_estimates all_estimates bestval best_exitflag ...
          best_grad allMaxVals] = ...
    solve_ml(starting_freeparams, allparams, free_index, ...
             observedData, ...
             numloops, numSmallLoops, ...
             numsimplex, numderiv, stddevs, printscreen, ...
             writetemp, display_simplex, display_deriv, restart_index)
%
% SOLVE_ML
%

% Greg Duffee
% September 2009, revised July 2011
%

%==============================================================
%
% To allow for interruptions and crashes, the code allows the
% search for an optimum to be restarted at a particular loop
% number.  This loop number, denoted 'restart_index', is passed to
% the routine.  If no loop number is passed, this value is set to
% one (which means that there is no restart).
%
%==============================================================

if (nargin<13); restart_index = 1; end;
if (restart_index > 1) 
  fprintf(1, 'Restarting at loop %d.\n', restart_index);
end

numfactors = 4; 

%==============================================================
%
% Set a flag that determines whether optimization is performed.
% Also set a flag that says whether this is a restart.  
%
%==============================================================

optimize = (numloops > 0) & ((numderiv + numsimplex) > 0); 
restart = (restart_index > 1); 

%==============================================================
%
% For the initial set of parameters passed to the routine,
% calculate the likelihood.  If no
% optimization is performed (or the optimization completely fails),
% this value is returned by the function.  Display the result
% to the screen unless this is a restart (in which case its 
% already been displayed to the screen during a previous run).  
%
%==============================================================

starting_allparams = poke_freeparams(starting_freeparams, allparams, ...
                     free_index); 
fval = kalman_only_loglike(starting_freeparams, starting_allparams, ...
                           free_index, observedData);

if ~restart
  fprintf(1, '\nInitial loglikelihood = %f\n', -fval); 
end

%==============================================================
%
% If the initial value of the likelihood is really, really, bad,
% Simplex is a good way to start the optimization.  Therefore, if
% this initial likelihood is extremely low, and as long as
% optimization is requested (i.e., numloops > 0), I use at 
% least one pass through Simplex, even if the input code didn't ask
% for one.
%
%==============================================================

if (optimize & (numsimplex == 0) & (fval > 1e8))
  fprintf(1, ['Poor initial point; using one iteration of simplex ' ...
              'to improve it.\n']);
  numsimplex = 1;
end

%==============================================================
%
% Set flags that determine what optimization routines are used.
%
%==============================================================

if optimize
  if (numsimplex > 0)
    use_simplex = 1;
  else
    use_simplex = 0;
  end
  if (numderiv > 0)
    use_deriv = 1; 
  else
    use_deriv = 0;
  end
end

%==============================================================
%
% Set initial values of the output.  
%
%==============================================================

free_estimates = starting_freeparams;
all_estimates = starting_allparams;
bestval = -fval; 
best_grad = [];

if ~optimize
  best_exitflag = 0; 
  allMaxVals = []; 
else
  best_exitflag = -10; 
  allMaxVals = -1d20*ones(numloops, 1); 
end
  
%==============================================================
%
% If any sets of starting parameters are to be generated randomly
% (this happens whenever more than one loop is used), set the 
% random number generator.  The current value of the seed is first
% read.  After completion of the optimization, the value of the
% seed is returned to this value.  (This happens at the end of the
% function.)  
%
%==============================================================

if (numloops > 1) 
  random_seed_hold = randn('state'); 
  ranseed = 0; 
end

%==============================================================
%
% If more than a single loop is used to find the optimum, I assume
% that we want a broad search to locate a good candidate region for
% the global optimum.  If a single loop is used, I assume that the
% search is starting from a point previously identified as a good
% region for the global optimum.  To increase the execution speed
% for the broad search, I can set the maximum number of Simplex
% iterations to a smaller number when 'numloops == 1' than
% otherwise.
%
%==============================================================

if (numloops == 1) 
  simplex_maxfunval = 3000;
  simplex_maxiter = 2800;
else
  simplex_maxfunval = 10000;
  simplex_maxiter = 8000;
end

%==============================================================
%
% Translate the input code that defines the requested level of
% verbosity for the optimization routines into the appropriate
% character strings. 
%
%==============================================================

if display_simplex
  simplex_display = char('iter');
else
  simplex_display = char('off'); 
end
if display_deriv
  deriv_display = char('iter');
else
  deriv_display = char('off'); 
end

%==============================================================
%
% Here starts the code that finds the optimum.  It loops through
% 'numloops' different starting values, using Simplex (if
% requested) and a derivative-based optimization routine (if
% requested).  
%
%==============================================================

if optimize

  for this_loop = 1:numloops

%==============================================================
%
% More detail is displayed on the screen for the first loop than
% the other loops.  (However, if 'printscreen' is set to zero,
% nothing is displayed.)
%
%==============================================================
    
    if (printscreen & (this_loop == 1) & (numloops > 1) ...
        & (~restart)) 
      fprintf(1, 'Using initial set of starting values...\n'); 
      fprintf(1, ['More detail is displayed for this first set ', ...
                  'than for the other sets.\n\n']);
    end

%==============================================================
%
% To allow for future restarts, I save the current value of the
% loop to disk.  If 'writetemp' is set to zero, I skip this
% step.
%
%==============================================================
        
    if writetemp
      f_output_id = fopen('tempindex.out', 'w'); 
      fprintf(f_output_id, '%d\n', this_loop);
      fclose(f_output_id); 
    end

%==============================================================
%
% Each loop uses a different starting value for the optimization.
% This starting value is denoted 'init_freeparams.'  
%
% The first loop uses the starting value passed to the routine, and
% all other loops use randomly generated starting values.  The
% randomly generated values are normally distributed, with means
% given by the starting value passed to the routine and standard
% deviations also passed to the routine.
%
%==============================================================

    if (this_loop == 1)
      init_freeparams = starting_freeparams; 
    else
      notvalid = 1; 
      while notvalid
        [init_freeparams newseed] = generate_random_freeparams( ...
            numfactors, starting_allparams, stddevs, free_index, ...
            ranseed); 
        ranseed = newseed; 
        funval = kalman_only_loglike(init_freeparams, ...
                                   starting_allparams, ...
                                   free_index, observedData);
        if funval < 1e7 
          notvalid = 0;
        end
      end
    end 

%==============================================================
%
% Calculate the value of the likelihood function at this starting
% value.  I do this in case the optimization crashes, so I have
% something to report for each loop.  For the initial loop, this
% duplicates a calculation made earlier.  (If there is a restart, I
% don't do this until the restart point.)  
%
%==============================================================

    if (this_loop >= restart_index) 
      fval = kalman_only_loglike(init_freeparams, ...
                                 starting_allparams, ...
                                 free_index, observedData);
      exitflag = -10; 
    end
    
%==============================================================
%
% Small loops run repeatedly through simplex and derivative-based
% estimation.
%
%==============================================================

    for thisSmallLoop = 1:numSmallLoops

%==============================================================
%
% Unless we are bypassing this loop because of a restart, go
% through 'numsimplex' iterations of Simplex to improve this 
% likelihood.
%
%==============================================================

      if (use_simplex & (this_loop >= restart_index)) 
        options = optimset('MaxFunEvals', simplex_maxfunval, ...
                           'MaxIter', simplex_maxiter, ...
                           'Display', simplex_display, ...
                           'TolFun', 1e-16, 'TolX', 1e-16); 
        for k = 1:numsimplex
          if (printscreen & (this_loop == 1)) 
            fprintf(1, 'In simplex, iteration %d...\n', k);
          end

%==============================================================
%
% If some weird error occurs within the Matlab optimization
% routines, I do not want the entire program to crash.  I just want
% to get out of that optimization routine and continue.  Therefore
% I place the Matlab simplex routine 'fminsearch' in a try-catch
% setting.  
%
% Note -- I have not experienced any crash of fminsearch with this
% likelihood function, but I put the try-catch structure here to
% mirror the code with fminunc below.  
%
%==============================================================

          try 
            [theseparams, fval, exitflag, output] = ...
                fminsearch(@(freeparams) ...
                           kalman_only_loglike(freeparams, ...
                                               starting_allparams, ...
                                               free_index, observedData), ...
                           init_freeparams, options);
            init_freeparams = theseparams;
          catch
            disp 'Oops -- an error occurred in executing fminsearch'
            exitflag = -10; 
          end
        
%==============================================================
%
% Print out some detail if this is the first loop.
%
%==============================================================

          if (printscreen & (this_loop == 1)) 
            if (exitflag == 1)
              fprintf(1,'    Convergence; likelihood = %13.5e\n', -fval); 
            end
            if (exitflag == 0)
              fprintf(1, ['    Max function evaluations or iterations ' ...
                          'reached; likelihood = %13.5e\n'], -fval); 
            end
            if (exitflag == -10)
              fprintf(1, 'Failure in fminsearch.\n');
            end
            fprintf(1, '\n'); 
          end
          
        end
      end

%==============================================================
%
% Unless we are bypassing this loop because of a restart, go
% through 'numderiv' iterations of a derivative-based routine to
% improve this likelihood.
%
%==============================================================
    
      if (use_deriv & (this_loop >= restart_index)) 
        % make sure 'grad' exists, in case a bad crash
        % terminates before the routine can calculate these values. 
        grad = [];
        % Set options for fminunc.  
        options = optimset('MaxFunEvals', 100000, 'MaxIter', 10000, ...
                           'LargeScale', 'off', ...
                           'Display', deriv_display, ...
                           'GradObj', 'off', 'DerivativeCheck', 'off', ...
                           'TolFun', 1e-12, 'TolX', 1e-12); 
        for k = 1:numderiv 
          if (printscreen & (this_loop == 1)) 
            fprintf(1, 'In derivative-based method, iteration %d...\n', k);
          end
          
          try 
            [theseparams, fval, exitflag, output, grad] = ...
                fminunc(@(freeparams) ...
                        kalman_only_loglike(freeparams, ...
                                            starting_allparams, ...
                                            free_index, observedData), ...
                        init_freeparams, options);
            init_freeparams = theseparams;
          catch
            disp 'Oops -- an error occurred in executing fminunc'
            exitflag = -10; 
          end

%==============================================================
%
% Print out some detail if this is the first loop.
%
%==============================================================

          if (printscreen & (this_loop == 1)) 
            if (exitflag == 1)
              fprintf(1, ['    Magnitude of gradient smaller than ', ...
                          'tolerance; likelihood = %13.5e\n'], -fval); 
            end
            if (exitflag == 2)
              fprintf(1, ['    Change in x is smaller than tolerance; ', ...
                          'likelihood = %13.5e\n'], -fval); 
            end
            if (exitflag == 3)
              fprintf(1, ['    Change in obj function is less than ', ...
                          'tolerance; likelihood = %13.5e\n'], -fval); 
            end
            if (exitflag == 0)
              fprintf(1, ['    Num of iterations or function evaluations ', ...
                          'exceeded; likelihood = %13.5e\n'], -fval); 
            end
            if (exitflag == -2)
              fprintf(1, ['    Line search cannot find acceptable ', ...
                          'point; likelihood = %13.5e\n'], -fval); 
            end
            if (exitflag == -10)
              fprintf(1, 'Failure in fminunc.\n');
            end
            fprintf(1, '\n'); 
          end
        end
        
        fprintf(1, 'current val = %13.5e\n', -fval); 
      
      end % ends small loop. 
      
%==============================================================
%
% After completing the derivative-based calculation of the optimum,
% save the ending gradient.  The ending value of the likelihood
% function is -fval.   
%
%==============================================================
      
      this_grad = -grad;
    
    end % this is the end of the if statement that checks whether the
        % derivative-based optimization is used.  

%==============================================================
%
% The loop is completed.  Display the optimum for this loop, then
% check whether the optimum is better than any previous optimum.
% If so, save the relevant values.
%
%==============================================================
    
    allMaxVals(this_loop) = -fval; 
    
    if (printscreen & (this_loop >= restart_index)) 
      fprintf(1, 'Starting vals #%d, likelihood = %13.5e\n', ...
              this_loop, -fval);
    end
    if ((this_loop >= restart_index) & ((-fval) > bestval))
      if (printscreen & (numloops > 1))
        fprintf(1, '  **Highest likelihood so far**\n'); 
      end
      bestval = -fval;  % save the likelihood value
      free_estimates = init_freeparams; % save the free parameters,
                                        % and in the line below,
                                        % save the entire parameter
                                        % vector.  
      all_estimates = poke_freeparams(free_estimates, ...
                                      starting_allparams, free_index);
      best_exitflag = exitflag; % save the exitflag 
      if (numderiv > 0) % save the gradient, if calculated.
        best_grad = this_grad;
      end

%==============================================================
%
% Save the best value to disk as a backup in case of a crash.
%
%==============================================================

      if writetemp 
        f_output_id = fopen('tempparams.out', 'w'); 
        for i = 1:length(free_estimates)
          fprintf(f_output_id, '%+14.6e\n', free_estimates(i));
        end
        fclose(f_output_id); 
      end
    end
  end % this is the end statement for the for loop across
      % numloops. 
  
end % this ends the if statement that determines whether any
    % optimization should be performed.  

  
%==============================================================
%
% Return the random number generator to its original seed.
%
%==============================================================

if (numloops > 1) 
  randn('state', random_seed_hold); 
end

return
