function [all_estimates bestval best_exitflag allMaxVals] = ...
    estimate_model(filename, printscreen, writetemp, ...
                   display_simplex, display_deriv, restart_index)
%
% ESTIMATE_MODEL : reads the control file 'filename' and estimates the
% term structure model.  Some intermediate results are printed to
% the screen if 'printscreen' == 1.  
%
% Greg Duffee
% September 2009
%

% assume this is not a restart if there is no restart index. 
if (nargin<6); restart_index = 1; end;

[inputfile outputfile] = read_control(filename);

[yielddatafile maturitydatafile ...
 numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro numUsedMacro usedMacroIdx ...
 numfactors firstobs lastobs macroLoadings ...
 use_ols_start use_pc factor_indexes ...
 numloops numSmallLoops numsimplex numderiv ...
 muStar_names muStar_flag muStar_initialval muStar_rand_stddev ...
 kStar_names kStar_flag kStar_initialval kStar_rand_stddev ...
 SigmaStar_names SigmaStar_flag SigmaStar_initialval ...
 SigmaStar_rand_stddev ...
 rinfty_name rinfty_flag rinfty_initialval rinfty_rand_stddev ...
 eig_names eig_flag eig_initialval eig_rand_stddev ...
 A_f_names A_f_flag A_f_initialval A_f_rand_stddev ...
 B_f_names B_f_flag B_f_initialval B_f_rand_stddev ...
 errorSD_names errorSD_flag errorSD_initialval errorSD_rand_stddev ...
] = read_input(inputfile);

numSeries = numUsedBonds + numUsedMacro; 

%
% don't bother writing temporary results if a single set of
% starting values is used.
%

writetemp_copy = writetemp; 
if (numloops < 2)
  writetemp_copy = 0;
end

[observedData maturities] = ...
    read_data(yielddatafile, maturitydatafile, ...
              macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs);


if (use_pc) 
  pcLoadings = pcacov(cov(observedData)); 
  Pmat = pcLoadings(:,1:numfactors)'; 
  Emat = pcLoadings(:,(numfactors+1):end)'; 
else
  other_indexes = setdiff((1:numSeries), factor_indexes); 
  Pmat = zeros(numfactors, numSeries);
  Emat = zeros(numSeries-numfactors, numSeries); 
  for i = 1:numfactors
    Pmat(i,factor_indexes(i)) = 1;
  end
  for i = 1:(numSeries - numfactors)
    Emat(i,other_indexes(i)) = 1;
  end
end

if (macroLoadings == 1)
  [freeparams bigparamvec free_index] = setup_initparams( ...
      muStar_initialval, kStar_initialval, SigmaStar_initialval, ...
      rinfty_initialval, eig_initialval, ...
      A_f_initialval, B_f_initialval, ...
      errorSD_initialval, muStar_flag, kStar_flag, ...
      SigmaStar_flag, rinfty_flag, eig_flag, A_f_flag, B_f_flag, ...
      errorSD_flag, ...
      observedData, use_ols_start, Pmat); 
else
  [freeparams bigparamvec free_index] = setup_initparamsStar( ...
      muStar_initialval, kStar_initialval, SigmaStar_initialval, ...
      rinfty_initialval, eig_initialval, ...
      A_f_initialval, B_f_initialval, ...
      errorSD_initialval, muStar_flag, kStar_flag, ...
      SigmaStar_flag, rinfty_flag, eig_flag, A_f_flag, B_f_flag, ...
      errorSD_flag, ...
      observedData, use_ols_start, Pmat); 
end

stddevs = pack_allparams( ...
    muStar_rand_stddev, kStar_rand_stddev, ...
    SigmaStar_rand_stddev, rinfty_rand_stddev, eig_rand_stddev, ...
    A_f_rand_stddev, B_f_rand_stddev, ...
    errorSD_rand_stddev, ... 
    muStar_flag, kStar_flag, SigmaStar_flag, rinfty_flag, ...
    eig_flag, A_f_flag, B_f_flag, errorSD_flag);

if (printscreen == 1) 

  fprintf(1, ['\n\nMaximum likelihood estimation of a ', ...
              'factor model\n']);
  fprintf(1,     ['-----------------------------------', ...
                  '-------------\n\n']);
  fprintf(1, 'Input file is  %s.\n', inputfile);
  fprintf(1, 'Output file is %s.\n', outputfile);
  fprintf(1, 'Data files are:\n');
  fprintf(1, '%s\n', yielddatafile);
  fprintf(1, '%s\n\n', macrodatafile);
  if (numloops == 1) 
    fprintf(1, ['A single set of starting parameters is used to ', ...
                'begin the optimization.\n']);
  end
  if (numloops > 1) 
    fprintf(1, ['%d sets of starting parameters are used to ', ...
                'begin the optimization.\n'], numloops);
    fprintf(1, 'All but the first are randomly generated.\n'); 
  end
  if (writetemp_copy == 1) 
    fprintf(1, ['Temporary results are written to tempparams.out ', ...
                'and tempindex.out.\n']); 
  else
    fprintf(1, 'Temporary results are not written anywhere.\n'); 
  end
  fprintf(1, '\n'); 
end

tolerance = 1e-12;

[funval loglikes filtered_values errors] = ...
    kalman_only_loglike(freeparams, bigparamvec, free_index, ...
                        observedData, maturities, numfactors, ...
                        macroLoadings, Pmat, tolerance);

[free_estimates all_estimates bestval best_exitflag ...
 best_grad allMaxVals] = ...
    solve_ml(freeparams, bigparamvec, free_index, ...
             observedData, maturities, macroLoadings, Pmat, ...
             tolerance, ...
             numloops, numSmallLoops, ...
             numsimplex, numderiv, stddevs, printscreen, ...
             writetemp_copy, display_simplex, display_deriv, ...
             restart_index); 

[funval loglikes filtered_values errors] = ...
    kalman_only_loglike(free_estimates, all_estimates, free_index, ...
                        observedData, maturities, numfactors, ...
                        macroLoadings, Pmat, tolerance);

%
% build up model vectors and matrices. 
%

[muStar, kStar, SigmaStar, rinfty, eigs, A_f, B_f, errorSDs] = ...
    unpack_allparams(numfactors, numUsedMacro, all_estimates);

[kMatStar SigmaMatStar B_fMat ...
          kEquivMatLatent maxAbsEquivEig H] = ...
    build_matrices(kStar, SigmaStar, B_f, eigs, errorSDs);

% Apply some normalizations.

SigmaStar_vec = SigmaStar;

for i = 1:numfactors
  SigmaMatStar(:,i) = sign(SigmaMatStar(i,i))*SigmaMatStar(:,i);
end
index = 0; 
for i = 1:numfactors
  for j = 1:i
    index = index + 1; 
    SigmaStar_vec(index) = SigmaMatStar(i,j);
  end
end

write_output(inputfile, outputfile, muStar, kStar, SigmaStar_vec, ...
             rinfty, eigs, A_f, B_f, errorSDs, ...
             bestval, best_exitflag)

return

