function [meanShortNom meanShortReal meanSlope ...
          meanRealSlope ...
          inflationComp consumptionComp mean5yrSpread ...
         mean5yrNom mean5yrReal] = ...
    modelFeatures(freeparams, allparams, free_index, ...
                  maturities, numFactors, numUsedMacro, ...
                  macroLoadings, Pmat)
%
% Greg Duffee
% November 2011
%
% Function returns the following, all in percent.  
%
% (1) unconditional mean one-quarter nominal yield, annualized 
% (2) unconditional mean one-quarter real yield, annualized
% (3) unconditional mean 20-quarter nominal yield, annualized 
% (4) unconditional mean 20-quarter real yield, annualized
% (5) Compensation per quarter for holding an asset that 
% has a beta of one with respect to inflation -- goes up by one
% percent when inflation goes up by one percent.  
% (6) Compensation per quarter for holding an asset that has a beta
% of one with respect to consumption growth -- goes up by one
% percent when consumption growth goes up by one percent.  
%


numbonds = length(maturities);

paramvec = poke_freeparams(freeparams, allparams, free_index); 

[muStar, kStar, SigmaStar, rinfty, eigs, A_f, B_f, errorSDs] = ...
    unpack_allparams(numFactors, numUsedMacro, paramvec);

[kMatStar SigmaMatStar B_fMat ...
          kEquivMatLatent maxAbsEquivEig H] = ...
    build_matrices(kStar, SigmaStar, B_f, eigs, errorSDs);


if (macroLoadings == 1) 
  [latentModel observableModel] = ...
      buildBothModels(muStar, kMatStar, SigmaMatStar, rinfty, ...
                      kEquivMatLatent, A_f, B_fMat, ...
                      Pmat, maturities); 
else
  [latentModel observableModel] = ...
      buildBothModelsStar(muStar, kMatStar, SigmaMatStar, rinfty, ...
                          kEquivMatLatent, A_f, B_fMat, ...
                          Pmat, maturities); 
end

mu = observableModel.mu;
Kmat = observableModel.K;
Sigma = observableModel.Sigma;
A = observableModel.A;
B = observableModel.B;

delta0 = observableModel.delta0;
delta1 = observableModel.delta1; 
muEquiv = observableModel.muEquiv;
KEquiv = observableModel.KEquiv; 

lambda0 = mu - muEquiv;
lambda1 = Kmat - KEquiv; 

meanState = inv(eye(4) - Kmat)*mu; 

maxHorizon = 20; 

bigBmat = createBigB(delta1, KEquiv, maxHorizon);
bigA = createBigA(delta0, muEquiv, Sigma, bigBmat);

priceBmat = bigBmat;
for i = 1:maxHorizon
  priceBmat(i,:) = -i*priceBmat(i,:);
end

meanShortNom = meanState(1)*400;
mean5yrNom = meanState(2)*400;

meanSlope = mean5yrNom - meanShortNom; 

%
% real 
%

Api = A(end-1); 
Bpi = B(end-1,:)';

Omegapi = H(end-1,end-1);

delta0Real = delta0 - Api - Bpi'*muEquiv + 0.5*Bpi'*Sigma*Sigma'*Bpi ...
    + 0.5*Omegapi;

delta1Real = delta1 - KEquiv'*Bpi;

lambda0Real = lambda0 - Sigma*Sigma'*Bpi;
lambda1Real = lambda1; 

bigBRealmat = createBigB(delta1Real, (Kmat - lambda1Real), maxHorizon);
bigAReal = createBigA(delta0Real, (mu - lambda0Real), Sigma, ...
                      bigBRealmat);

meanRealTS = 400*(bigAReal + bigBRealmat*meanState);

mean5yrReal = meanRealTS(20);

meanShortReal = 400*(meanState(1) - meanState(3) + ...
                     Bpi'*(lambda0Real + lambda1Real*meanState) + ...
                     0.5*Bpi'*Sigma*Sigma'*Bpi + 0.5*Omegapi);

mean5yrSpread = 400*((bigA(20) - bigAReal(20)) + ...
    (bigBmat(20,:) - bigBRealmat(20,:))*meanState);

meanRealSlope = mean5yrReal - meanShortReal; 

%
% unconditional mean compensation for facing shocks to inflation
% and aggregate consumption growth.  
%

unconLambda = lambda0 + lambda1*meanState;

inflationComp = 100*unconLambda(end-1);
consumptionComp = 100*unconLambda(end); 


return
