function output = ols_robust(Y, passed_X, use_constant, lags, use_newey_west)
% OLS_ROBUST : OLS estimation with both classical and robust standard errors.
%              Tests of the null that all coefficients equal zero (except
%              for a constant term) are also reported.  An R^2 is reported
%              when a constant term is included in the regression.  
% Greg Duffee
% November 2005
%
% The regression is either 
%
%     Y = passed_X \beta + epsilon 
%
% or, if use_constant == 1, 
%
%     Y = [1 passed_X] \beta + epsilon
% 
% If lags=0, the routine constructs both OLS (strict exogeneity) and 
% White covariance matrices for the parameter estimates.  The other
% covariance matrices are set to empty values.  
%
% If lags>0, the routine constructs both Hansen-Hodrick and Hansen 
% covariance matrices.  (They differ because Hansen-Hodrick is not robust 
% to generalized heteroskedasticity).  For the Hansen covariance matrix, 
% there is an option for a Newey-West weighting matrix.  Other covariance
% matrices are set to empty values.  
%

T = size(Y,1); 

if (use_constant == 1)
  X = [ones(T,1) passed_X];
else
  X = passed_X;
end

k = size(X,2); 
R_matrix = eye(k);
if (use_constant == 1) 
  R_matrix = R_matrix(2:end,:); 
end

%
% get OLS parameter estimates with Matlab routine.
%
[beta temp resids] = regress(Y, X); 
joint_val = R_matrix * beta; 
dim_test = length(joint_val); 
if (use_constant == 0)
  rsqr = [];
  adjrsqr = []; 
else
  rsqr = 1 - (resids'*resids) / ...
            ((Y - mean(Y))' * (Y - mean(Y)));
  adjrsqr = 1 - (1 - rsqr)*(T-1)/(T-k);
end

%
% set up moment matrix
%

moments = repmat(resids, 1, k) .* X;

%
% Estimate of covariance matrix of contemporaneous moments.
%

S = (1/T) * moments' * moments; 
XprimeX_inv = invpd(X'*X); % note, uses my invpd 

%
% The case of lags == 0.
%

if (lags == 0) 
  
  s_squared_hat = (1/(T-k)) * resids'*resids; 
  ols_vcv = XprimeX_inv * s_squared_hat; 
  white_vcv = T * XprimeX_inv * S * XprimeX_inv; 
  hh_vcv = [];
  hansen_vcv = []; 
  ols_tstat = beta ./ sqrt(diag(ols_vcv)); 
  white_tstat = beta ./ sqrt(diag(white_vcv)); 
  hh_tstat = [];
  hansen_tstat = []; 
  hh_test = [];
  hh_pval = [];
  hansen_test = [];
  hansen_pval = []; 
  ols_test = joint_val' * ...
      invpd(R_matrix * ols_vcv * R_matrix') * ...
      joint_val /  dim_test; 
  ols_pval = 1.0 - fcdf(ols_test, dim_test, (T-k)); 
  white_test = joint_val' * ...
      invpd(R_matrix * white_vcv * R_matrix') * ...
      joint_val; 
  white_pval = 1.0 - chi2cdf(white_test, dim_test); 
else
  
%
% The case of lags > 0. 
%

  ols_vcv = [];
  white_vcv = []; 
  ols_tstat = [];
  white_tstat = []; 
  ols_test = [];
  ols_pval = [];
  white_test = [];
  white_pval = []; 
  for j = 1:lags
    F.matrix = (1/T) * moments(1:(end-j),:)' * ...
        moments((1+j):end,:); 
    if (use_newey_west == 1)
      S = S + (1 - (j/(lags+1))) * (F.matrix + F.matrix'); 
    else
      S = S + (F.matrix + F.matrix'); 
    end
  end

  hansen_vcv = T * XprimeX_inv * S * XprimeX_inv; 
  hansen_tstat = beta ./ sqrt(diag(hansen_vcv)); 
  
  big_hh_matrix = zeros(T,T); 
  for j = 0:lags
    thisR = (1/T) * resids(1:(end-j))' * resids((1+j):end); 
    for jj = 1:(T - j)
      big_hh_matrix(jj, jj + j) = thisR;  
      big_hh_matrix(jj+j,jj) = thisR;  
    end
  end
  hh_vcv = XprimeX_inv * X' * big_hh_matrix * ...
           X * XprimeX_inv;   
  hh_tstat = beta ./ sqrt(diag(hh_vcv)); 
  
  [invmat okinverse] = invpd(R_matrix * hh_vcv * R_matrix');
  if (okinverse == 1)
    hh_test = joint_val' * invmat * joint_val; 
    hh_pval = 1.0 - chi2cdf(hh_test, dim_test); 
  else
    hh_test = NaN; 
    hh_pval = NaN; 
  end
  [invmat okinverse] = invpd(R_matrix * hansen_vcv * R_matrix');
  if (okinverse == 1)
    hansen_test = joint_val' * invmat * joint_val; 
    hansen_pval = 1.0 - chi2cdf(hansen_test, dim_test); 
  else
    hansen_test = NaN; 
    hansen_pval = NaN; 
  end
end

output.T = T; 
output.beta = beta;
output.rsqr = rsqr;
output.adjrsqr = adjrsqr; 
output.ols_vcv = ols_vcv;
output.white_vcv = white_vcv;
output.hh_vcv = hh_vcv;
output.hansen_vcv = hansen_vcv;
output.ols_tstat = ols_tstat; 
output.white_tstat = white_tstat; 
output.hh_tstat = hh_tstat; 
output.hansen_tstat = hansen_tstat; 
output.ols_test = ols_test;
output.ols_pval = ols_pval;
output.white_test = white_test;
output.white_pval = white_pval;
output.hh_test = hh_test;
output.hh_pval = hh_pval;
output.hansen_test = hansen_test;
output.hansen_pval = hansen_pval;

return

