function [covMatrix valid covDeriv] = uncon_vcv_general(F, Sigma)
% UNCON_VCV_GENERAL constructs the unconditional covariance for x_t
% where 
% x_t = F x_{t-1} + e_t, var(e_t) = Sigma*Sigma'.  
% The math follows Hamilton, page 265.  'valid' is set to zero if
% the unconditional covariance matrix implies nonstationary
% dynamics. The matrix Sigma is assumed to be lower triangular.   
%
%
% Greg Duffee
% March 2005
% revised Feb 2008 to add derivatives.  
%
% Order of parameters is [vec(F); Sigma(1,1); Sigma(2,1:2)'; ...]
%
%==============================================================

% First check to see if the process is stationary.  If not, bail
% out.  

d = eig(F); 
if (max( d .* conj(d)) >= 1)
  valid = 0; 
  covMatrix = []; 
  covDeriv = []; 

else

  valid = 1; 
  n = size(F,1); 
  dimSqr = n^2;
  dimParams = dimSqr + n*(n+1)/2;
  Q = Sigma*Sigma'; 
  invMat = inv( eye(dimSqr) - kron(F,F) ); 
  vec_sigma = invMat * vec(Q);
  covMatrix = vec_inverse(vec_sigma);

  % if first derivatives are required, compute them.  
  
  if (nargout > 2) 
    
    % initialize some values 
    kronDeriv = zeros(dimSqr,dimSqr,dimParams); 
    invDeriv = zeros(dimSqr,dimSqr,dimParams); 
    vecCovDeriv = zeros(dimSqr,dimParams); 
    covDeriv = zeros(n,n,dimParams); 

    % I build these derivatives up slowly.  First, compute the
    % derivative of (I - kron(F,F)).  
    
    for j = 1:n
      for i = 1:n
        paramPos = (j-1)*n + i; 
        % following term is deriv of f(i,j)*F wrt f(i,j)
        kronDeriv(((i-1)*n+1):(i*n),((j-1)*n+1):(j*n), ...
                  paramPos) = -F; 
        % following terms are deriv of f(p,q)*F wrt F(i,j) term of
        % F. 
        Z = zeros(n,n);
        Z(i,j) = 1; 
        for p=1:n
          for q = 1:n
            kronDeriv(((p-1)*n+1):(p*n),((q-1)*n+1):(q*n), ...
                      paramPos) = ...
                kronDeriv(((p-1)*n+1):(p*n),((q-1)*n+1):(q*n), ...
                          paramPos) - F(p,q)*Z; 
          end
        end
      end
    end
    
    % Using this derivative, compute the derivative of the inverse
    % of (I-kron(F,F))
    
    for j = 1:n
      for i = 1:n 
        paramPos = (j-1)*n + i; 
        for p = 1:dimSqr
          for q = 1:dimSqr
            Z = zeros(dimSqr,dimSqr);
            Z(p,q) = 1; 
            prodTerm = -invMat*Z*invMat; 
            invDeriv(:,:,paramPos) = invDeriv(:,:,paramPos) + ...
                prodTerm*kronDeriv(p,q,paramPos); 
          end
        end
      end
    end
    
    % Now compute the derivative of Q wrt the elements of Sigma.  

    Qderivs = zeros(n,n,dimParams); 
    index = dimSqr;
    for i = 1:n
      for j = 1:i
        index = index + 1; 
        Z = zeros(n,n);
        Z(i,j) = 1; 
        Qderivs(:,:,index) = (Z*Sigma' + Sigma*Z');
      end
    end
    
    % Compute the derivative of vec (inv( I - kron(F,F)) * vec(Q) )
    % in two steps.  First handle parameters of F, then parameters
    % of Sigma.  I do this because the parameters separate into
    % those that affect inv(I - kron(F,F)) and those that affect
    % Q. 
  
    vecQ = vec(Q); 
    for paramPos = 1:dimSqr
      vecCovDeriv(:,paramPos) = invDeriv(:,:,paramPos) * vecQ;
    end
  
    % Now derivatives wrt parameters of Sigma.
  
    for paramPos = (1+dimSqr):dimParams
      vecCovDeriv(:,paramPos) = invMat * vec(Qderivs(:,:,paramPos));
    end

    % unpack the vector into derivatives of the covariance matrix. 
    
    for paramPos = 1:dimParams
      covDeriv(:,:,paramPos) = vec_inverse(vecCovDeriv(:, ...
                                                       paramPos));
    end
    
  end % end of if statement that checks if derivatives are to be
      % computed.  
    
end % end of if statement that checks for stationarity. 

return
