% Greg Duffee
%
% Construct measures of the output gap.
% Also construct measures of the growth of economic activity. 
%

%=======================================================================

clear
close all

%=======================================================================

outputFile = '..\excelFiles\cycleActivity_Q_Measures.xls';

%=======================================================================

quarterlyData = xlsread('..\excelFiles\quarterlydata.xls', ...
                        'quarterlyData', 'A13:Y264');

date = quarterlyData(:,1);
nondurServPerCap = quarterlyData(:,2);
realGDP = quarterlyData(:,3); 
ip = quarterlyData(:,4);
CFNAI = quarterlyData(:,5);
CFNAI_3m = quarterlyData(:,6);
capUtil = quarterlyData(:,7);
LNrealAct = quarterlyData(:,8);

yield_3m = quarterlyData(:,13); 

%=======================================================================
%
% Construct quarterly cyclical measures.  They are constructed from
% real GDP, industrial production, and capacity utilization.
%
%=======================================================================


% cycle measures are detrended log GDP, detrended log IP, HP
% filtered log GDP, HP filtered log IP, demeaned industrial
% capacity. 

cycleMeasures = NaN*zeros(length(yield_3m), 5);


%----------------------------------------------------------------
%
% Get cycles from GDP and industrial production in two ways.
% First, remove a linear trend from log values.  Second, remove the
% trend using an HP filter.  Do this for the sample 1952Q2 through
% 2010Q4, aligning the sample with yields.
% 
%----------------------------------------------------------------

OKsamp = isfinite(yield_3m);
T = sum(OKsamp);
constantTerm = ones(T, 1);
timeTrend = (1:T)';

logGDP = log(realGDP);
logIP = log(ip); 

output = ols(logGDP(OKsamp), [constantTerm timeTrend]);
detrendLogGDP = output.resid;

output = ols(logIP(OKsamp), [constantTerm timeTrend]);
detrendLogIP = output.resid;

w = 1600; 
smoothedLogGDP = hpfilter(logGDP(OKsamp),w);
HPrealGDP = logGDP(OKsamp) - smoothedLogGDP;

smoothedLogIP = hpfilter(logIP(OKsamp),w);
HPlogIP = logIP(OKsamp) - smoothedLogIP;

cycleMeasures(OKsamp,1) = detrendLogGDP;
cycleMeasures(OKsamp,2) = detrendLogIP;
cycleMeasures(OKsamp,3) = HPrealGDP;
cycleMeasures(OKsamp,4) = HPlogIP;

%----------------------------------------------------------------
%
% Get cycle from capacity utilization by removing the mean.
% 
%----------------------------------------------------------------

demeanCapUtil = capUtil - nanmean(capUtil);

cycleMeasures(:,5) = demeanCapUtil;

%=======================================================================
%
% construct quarterly activity growth measures.  They are both raw
% growth rates and arma(1,1) filtered growth rates.  
%
%=======================================================================

activityMeasures = NaN*zeros(length(yield_3m), 6);

cGrowth = [NaN; diff(log(nondurServPerCap))];
ipGrowth = [NaN; diff(log(ip))];
gdpGrowth = [NaN; diff(log(realGDP))];

activityMeasures(:,1) = cGrowth;
activityMeasures(:,2) = gdpGrowth;
activityMeasures(:,3) = ipGrowth;

% arma(1,1) filtered, to match the sample 1952Q2 through 2010Q4.
% Use one-step-ahead forecasts as current values.  

tmpOKsamp = isfinite([repmat(NaN, 16, 1); ones((252-16),1)]);

constant = 1; 
ar = 1;
ma = 1; 

y = cGrowth(tmpOKsamp); 
[parameters errors] = armaxfilter(y,constant,ar,ma); 
cGrowthARMA = parameters(1) + parameters(2)*y(2:end) + ...
    parameters(3)*errors;
y = gdpGrowth(tmpOKsamp); 
[parameters errors] = armaxfilter(y,constant,ar,ma); 
gdpGrowthARMA = parameters(1) + parameters(2)*y(2:end) + ...
    parameters(3)*errors;
y = ipGrowth(tmpOKsamp); 
[parameters errors] = armaxfilter(y,constant,ar,ma); 
ipGrowthARMA = parameters(1) + parameters(2)*y(2:end) + ...
    parameters(3)*errors;

activityMeasures(OKsamp,4) = cGrowthARMA;
activityMeasures(OKsamp,5) = gdpGrowthARMA;
activityMeasures(OKsamp,6) = ipGrowthARMA;

%---------------------------------------------------------------------%
%
%  Write quarterly data file.
%
%---------------------------------------------------------------------%

writeData = [date cycleMeasures activityMeasures];

header1 = {['1.  Log real GDP, constant and time trend removed for 1952Q2 ' ...
            'to 2010Q4.']};
header2 = {'2. Log IP, transformed in same way.'};
header3 = {['3.  Log real GDP, residual from Hodrick-Prescott filter ' ...
            'for 1952Q2 through 2010Q4']};
header4 = {'4.  Log IP, transformed in same way.'};
header5 = {'5. Capacity utilization, demeaned for available sample.'};
header6 = {['6. Log-differenced real per capita consumption on ' ...
            'nondurables and services']};
header7 = {'7. Log-differenced real GDP.'};
header8 = {'8. Log-differenced IP'};
header9 = {['9. One-q-ahead forecast of log-differenced real per ' ...
            'capita C, from ARMA(1,1) model estimated for 1952Q1 through ' ...
             '2010Q4']};
header10 = {['11. One-q-ahead forecast of log-differenced real GDP, ' ...
            'from ARMA(1,1) model estimated for 1952Q1 through ' ...
             '2010Q4']};
header11 = {['10. One-q-ahead forecast of log-differenced IP, ' ...
            'from ARMA(1,1) model estimated for 1952Q1 through ' ...
             '2010Q4']};

header = {'Date', '1', '2', '3', '4', '5', '6', '7', '8', '9', ...
          '10', '11'};

xlswrite(outputFile, header1,  'quarterlyData',  'A1');
xlswrite(outputFile, header2,  'quarterlyData',  'A2');
xlswrite(outputFile, header3,  'quarterlyData',  'A3');
xlswrite(outputFile, header4,  'quarterlyData',  'A4');
xlswrite(outputFile, header5,  'quarterlyData',  'A5');
xlswrite(outputFile, header6,  'quarterlyData',  'A6');
xlswrite(outputFile, header7,  'quarterlyData',  'A7');
xlswrite(outputFile, header8,  'quarterlyData',  'A8');
xlswrite(outputFile, header9,  'quarterlyData',  'A9');
xlswrite(outputFile, header10,  'quarterlyData',  'A10');
xlswrite(outputFile, header11,  'quarterlyData',  'A11');

xlswrite(outputFile, header, 'quarterlyData', 'A14:L14');

xlswrite(outputFile, writeData, ...
         'quarterlyData', 'A15:L266');

