% Greg Duffee
%
% Set up data for use in monthly forecasting regressions.  Aligning
% the data properly is a big issue here.  All data are padded or
% truncated to begin with June 1952 and end with December 2010.
% This is 703 months.  
%

%=======================================================================

clear
close all

%=======================================================================

% file names

monthFile = char('..\excelFiles\monthlydata.xls');
varianceFile = char('..\excelFiles\realizedVar.xls'); 
LudvigsonNgFile = char('..\matFile\Fhat64'); 

outputFile = char('..\excelFiles\monthlyPredictiveVars.xls');

monthPage = char('monthlyData'); 
variancePage = char('RealizedVariances'); 

%=======================================================================

% load data

IPdata = xlsread(monthFile, monthPage, 'B13:B768');
% start date January 1948

yieldData = xlsread(monthFile, monthPage, 'D66:i768'); 
% start date June 1952

datesData = xlsread(monthFile, monthPage, 'A66:A768');
% start date June 1952

varianceData = xlsread(varianceFile, variancePage, 'B24:C618');
% start date June 1961


load(LudvigsonNgFile); 
LNfactors = Fhat_T; 
% Data range from 1964:1 to 2007:12

%=======================================================================

% Get ready to align data 
%

numValidMonths = size(yieldData, 1); 


%=======================================================================

% Slope of term structure.

slope = yieldData(:,end) - yieldData(:,1); 

%=======================================================================

% Construct forward rates from CRSP yields of one through five
% years.  

y1y = 1200*yieldData(:,2);
y2y = 1200*yieldData(:,3);
y3y = 1200*yieldData(:,4);
y4y = 1200*yieldData(:,5);
y5y = 1200*yieldData(:,6);

bigT = length(y1y);

% here are the forward rates

f2y = 2*y2y - y1y;
f3y = 3*y3y - 2*y2y;
f4y = 4*y4y - 3*y3y;
f5y = 5*y5y - 4*y4y;

% stack them into a matrix, including the one-year yield.

forwardRates = [y1y f2y f3y f4y f5y];

% there is no need to align this matrix with the proper sample --
% it is already set up.  

%=======================================================================

% Construct an average of the forward rates from month t-11 to
% month t.  

avgForwardRates = [NaN*zeros(11,5); zeros(bigT-11, 5)];
for i = 0:11
  avgForwardRates(12:end,:) = avgForwardRates(12:end,:) + ...
      forwardRates((1+i):(end-11+i),:)/12;
  
end

% again, no need to align these data with the proper sample.  

%=======================================================================

% Construct log change of IP.

diffLogIP = [NaN; log(IPdata(2:end)) - log(IPdata(1:(end-1)))];

% Construct a 12-month average of log change in IP.

avgIPgrowth = [NaN*zeros(12,1); ...
               (log(IPdata(13:end)) - log(IPdata(1:(end-12))))/12];

% truncate data to begin with June 1952.  

diffLogIP = diffLogIP((end-numValidMonths+1):end);

avgIPgrowth = avgIPgrowth((end-numValidMonths+1):end);

%=======================================================================

% Construct conditional standard deviations of one-month-ahead
% innovations in yields on long-maturity bonds.  
% Do this by projecting realized SDs on lagged realized SDs and lagged term
% structures.  
%

numLagsRealizedSDs = 9; 
numLagsYieldInfo = 1;

% square root to get realized standard deviations, while padding
% the data series. 

realizedLongMatSDs = ...
    [repmat(NaN, numValidMonths - size(varianceData,1), 1); ...
     sqrt(varianceData(:,2))];

y5y = yieldData(:,end); 
y1y = yieldData(:,1); 
FBslope = y5y - y1y;
yieldInfo = [y5y FBslope]; 

laggedRealizedSDs = [];
laggedYieldInfo = []; 

for i = 1:numLagsRealizedSDs
  laggedRealizedSDs = [laggedRealizedSDs ...
                      lag(realizedLongMatSDs, i, NaN)];
end

for i = 1:numLagsYieldInfo
  laggedYieldInfo = [laggedYieldInfo lag(yieldInfo, i, NaN)];
end

xmat = [ones(numValidMonths,1) laggedRealizedSDs laggedYieldInfo];
validObs = sum(isfinite(xmat), 2) == size(xmat, 2);

regResults = ols(realizedLongMatSDs(validObs), xmat(validObs,:));

% This regression gives us the coefficients for constructing
% conditional standard deviations.  Now set up the explanatory
% variables to construct conditional standard deviations.  


currentRealizedSDs = realizedLongMatSDs;
currentYieldInfo = yieldInfo; 

for i = 1:(numLagsRealizedSDs-1)
  currentRealizedSDs = [currentRealizedSDs ...
                      lag(realizedLongMatSDs, i, NaN)];
end

for i = 1:(numLagsYieldInfo-1)
  currentYieldInfo = [currentYieldInfo lag(yieldInfo, i, NaN)];
end

xmat = [ones(numValidMonths,1) currentRealizedSDs currentYieldInfo];

condSDs = xmat * regResults.beta; 

%=======================================================================

% Pad the Ludvigson-Ng principal components.  Also change the sign
% of the first component.
%

[lengthLN numLN] = size(LNfactors);

paddedLNfactors = [ repmat(NaN, numValidMonths - lengthLN - 36, numLN); ...
    [-LNfactors(:,1) LNfactors(:,2:end)]; ...
    repmat(NaN, 36, numLN)];


%---------------------------------------------------------------------%
%
%  Write data file of forecasting variables
%
%---------------------------------------------------------------------%

writeData = [datesData slope forwardRates avgForwardRates diffLogIP ...
             avgIPgrowth condSDs paddedLNfactors];

header1 = {'1. Slope of yield (5yr less 3mon).'}; 
header2 = {'2. One year yield'};
header3 = {'3. Forward rate from year 1 to year 2.'};
header4 = {'4. Forward rate from year 2 to year 3.'};
header5 = {'5. Forward rate from year 3 to year 5.'};
header6 = {'6. Forward rate from year 4 to year 6.'};
header7 = {'7. 12-month average of (2).'};
header8 = {'8. 12-month average of (3).'};
header9 = {'9. 12-month average of (4).'};
header10 = {'10. 12-month average of (5).'};
header11 = {'11. 12-month average of (6).'};
header12 = {'12. Log-diff industrial production.'};
header13 = {'13. 12-month average of (12).'};
header14 = {['14. Conditional SD of next month long-maturity ' ...
             'yields.']};
header15 = {'15. First PC, Ludvigson-Ng (changed sign).'};
header16 = {'16-22.  PCs two through eight.'}; 

header = {'Date', '1', '2', '3', '4', '5', '6', '7', '8', '9', ...
          '10', '11', '12', '13', '14', '15', '16', '17', '18', ...
         '19', '20', '21', '22'};

xlswrite(outputFile, header1,  monthPage,  'A1');
xlswrite(outputFile, header2,  monthPage,  'A2');
xlswrite(outputFile, header3,  monthPage,  'A3');
xlswrite(outputFile, header4,  monthPage,  'A4');
xlswrite(outputFile, header5,  monthPage,  'A5');
xlswrite(outputFile, header6,  monthPage,  'A6');
xlswrite(outputFile, header7,  monthPage,  'A7');
xlswrite(outputFile, header8,  monthPage,  'A8');
xlswrite(outputFile, header9,  monthPage,  'A9');
xlswrite(outputFile, header10,  monthPage,  'A10');
xlswrite(outputFile, header11,  monthPage,  'A11');
xlswrite(outputFile, header12,  monthPage,  'A12');
xlswrite(outputFile, header13,  monthPage,  'A13');
xlswrite(outputFile, header14,  monthPage,  'A14');
xlswrite(outputFile, header15,  monthPage,  'A15');
xlswrite(outputFile, header16,  monthPage,  'A16');

xlswrite(outputFile, header, monthPage, 'A18:W18');

xlswrite(outputFile, writeData, ...
         monthPage, 'A19:W721');


