function output = covarFunction(freeparams, allparams, free_index, ...
                                numFactors, numObserved, horizon, ...
                                beta, gamma)
%
%
% Greg Duffee
% June 2011
%
% requires first element of observables to be the nominal short
% rate, the third-to-last element to be the five-year bond yield, 
% the second-to-last element of observables to be inflation,
% and last element to be consumption growth.  
%

idx_3mon = 1;
idx_5yr = numObserved - 2;
idx_pi = numObserved - 1;
idx_cgrowth = numObserved; 

paramvec = poke_freeparams(freeparams, allparams, free_index); 

[mu K_vec Sigma_vec A B_vec errorSDs] = ...
    unpack_allparams(numFactors, numObserved, paramvec);
[Kmat Sigma B H] = ...
    build_matrices(K_vec, Sigma_vec, B_vec, errorSDs); 


tmpM = inv(eye(numFactors) - beta*Kmat); % infinite horizon sum
                                         % discounted at beta. 
tmp2 = B(idx_cgrowth,:)*tmpM; % consumption loading on this.  
tmp3 = B(idx_cgrowth,:)*(eye(numFactors) - (1 - gamma)*tmpM); % risk 
%                        compensation component under recursive utility

macro_real_delta0 = - log(beta) + A(idx_cgrowth) + ...
    B(idx_cgrowth,:) * mu  ...
    + 0.5*(1 - gamma)^2 * ( tmp2*Sigma*Sigma'*tmp2' + ...
                            H(idx_cgrowth,idx_cgrowth) ) ...
    - 0.5*( (gamma^2)*H(idx_cgrowth,idx_cgrowth) ...
            + tmp3*Sigma*Sigma'*tmp3' );
macro_real_delta1 = (B(idx_cgrowth,:)*Kmat)';
macro_nom_delta0 = macro_real_delta0 + A(idx_pi) + B(idx_pi,:)*mu - ...
    0.5*B(idx_pi,:)*Sigma*Sigma'*B(idx_pi,:)' - 0.5*H(idx_pi,idx_pi) - ...
    tmp3*Sigma*Sigma'*B(idx_pi,:)';
macro_nom_delta1 = (macro_real_delta1' + B(idx_pi,:)*Kmat)';

macro_real_lambda0 = Sigma*Sigma'*tmp3';
macro_nom_lambda0 = macro_real_lambda0 + Sigma*Sigma'*B(idx_pi,:)';

lambda1 = 0;

real_bigBmat = createBigB(macro_real_delta1, Kmat, horizon);
real_bigAvec = createBigA(macro_real_delta0, (mu - macro_real_lambda0), ...
                          Sigma, real_bigBmat);

nom_bigBmat = createBigB(macro_nom_delta1, Kmat, horizon);
nom_bigAvec = createBigA(macro_nom_delta0, (mu - macro_nom_lambda0), ...
                          Sigma, nom_bigBmat);

stateMean = inv(eye(numFactors) - Kmat)*mu; 

macro_meanRealTS = real_bigAvec + real_bigBmat*stateMean;
macro_meanNomTS = nom_bigAvec + nom_bigBmat*stateMean;

riskPremiumHorizonPlus1 = ...
    (-horizon*nom_bigBmat(horizon,:))*macro_nom_lambda0;

%
% covariance of two pieces of nominal short rate with three pieces
% of the nominal stochastic discount factor.  The two short rate
% pieces are the innovations in the real rate and one-step-ahead
% inflation.  The three
% pieces of the nominal SDF are the innovation in immediate
% consumption growth, the innovation in the PV of expected future
% consumption growth, and the innovation in inflation.
%

sumVector = B(idx_cgrowth,:)*tmpM*Sigma; 

avgState = 0; 
for j = 1:horizon
  avgState = avgState + Kmat^j;
end
avgState = avgState/horizon;
  
meanPiResponse = B(idx_pi,:)*avgState;
meanCResponse = B(idx_cgrowth,:)*avgState; 
  
output.covCmeanPi = B(idx_cgrowth,:)*Sigma*Sigma'*meanPiResponse';
output.covCmeanC = B(idx_cgrowth,:)*Sigma*Sigma'*meanCResponse';
output.covSUMmeanPi = sumVector*Sigma'*meanPiResponse';
output.covSUMmeanC = sumVector*Sigma'*meanCResponse';
output.covPimeanPi = B(idx_pi,:)*Sigma*Sigma'*meanPiResponse';
output.covPimeanC = B(idx_pi,:)*Sigma*Sigma'*meanCResponse';
output.covCNomBondYield = ...
    B(idx_cgrowth,:)*Sigma*Sigma'*nom_bigBmat(horizon,:)';
output.covCRealBondYield = ...
    B(idx_cgrowth,:)*Sigma*Sigma'*real_bigBmat(horizon,:)';
output.covCActNom3mYield = B(idx_cgrowth,:)*Sigma*Sigma'*B(idx_3mon,:)';
output.covCActNom5yrYield = B(idx_cgrowth,:)*Sigma*Sigma'*B(idx_5yr,:)';
output.covSUMNomBondYield = sumVector*Sigma'*nom_bigBmat(horizon,:)';
output.covSUMRealBondYield = sumVector*Sigma'*real_bigBmat(horizon,:)';
output.covSUMActNom3mYield = sumVector*Sigma'*B(idx_3mon,:)';
output.covSUMActNom5yrYield = sumVector*Sigma'*B(idx_5yr,:)';
output.covPiNomBondYield = B(idx_pi,:)*Sigma*Sigma'*nom_bigBmat(horizon,:)';
output.covPiRealBondYield = B(idx_pi,:)*Sigma*Sigma'*real_bigBmat(horizon,:)';
output.covPiActNom3mYield = B(idx_pi,:)*Sigma*Sigma'*B(idx_3mon,:)';
output.covPiActNom5yrYield = B(idx_pi,:)*Sigma*Sigma'*B(idx_5yr,:)';
output.riskPremiumHorizonPlus1 = riskPremiumHorizonPlus1;

output.macro_meanRealTS = macro_meanRealTS;
output.macro_meanNomTS = macro_meanNomTS;

return
