function output = regressFun(freeparams, allparams, free_index, ...
                                numFactors, numObserved)
%
%
% Greg Duffee
% June 2011
%
% requires first element of observables to be the nominal short
% rate, the third-to-last element to be the five-year bond yield, 
% the second-to-last element of observables to be inflation,
% and last element to be consumption growth.  
%

%============================================================
%
% Population regressions of nominal bond yields.
%
%=========================================================== 


idx_pi = numObserved - 1;
idx_cgrowth = numObserved; 
idx_3mon = 1;

if (numObserved == 4) 
  idx_5yr = numObserved - 2;
end

paramvec = poke_freeparams(freeparams, allparams, free_index); 

[mu K_vec Sigma_vec A B_vec errorSDs] = ...
    unpack_allparams(numFactors, numObserved, paramvec);
[Kmat Sigma B H] = ...
    build_matrices(K_vec, Sigma_vec, B_vec, errorSDs); 

[stateCovMat valid] = uncon_vcv_general(Kmat, Sigma); 

% Regress actual nominal short rate on two components of 
% model-implied nominal short rate

expNextPiLoading = B(idx_pi,:)*Kmat;
expNextCLoading = B(idx_cgrowth,:)*Kmat;
nom3MonLoad = B(idx_3mon,:);

if (numObserved == 4) 
  nom5YrLoad = B(idx_5yr,:);
end

% 20-quarter-ahead avg of K's raised to the period. 
tmpMat = (1/20)*inv(eye(numFactors) - Kmat)*(eye(numFactors) - Kmat^20);

expLongPiLoading = expNextPiLoading * tmpMat;
expLongCLoading = expNextCLoading * tmpMat;
expLong3MLoading = nom3MonLoad * tmpMat; 

% matrices of c and pi expectations.

nextLoading = [expNextCLoading; expNextPiLoading];
longLoading = [expLongCLoading; expLongPiLoading];

% covariances with explanatory variables

cov3Mnext = nom3MonLoad * stateCovMat * nextLoading';

if (numObserved == 4) 
  cov5YRlong = nom5YrLoad * stateCovMat * longLoading';
  cov5YRlong3m = nom5YrLoad * stateCovMat * expLong3MLoading';
end

% variances of explanatory variables

varNext = nextLoading * stateCovMat * nextLoading';
varLong = longLoading * stateCovMat * longLoading';
varLong3m = expLong3MLoading * stateCovMat * expLong3MLoading';

% variances of dependent variables, including idiosyncratic component

var3M = nom3MonLoad * stateCovMat * nom3MonLoad' + H(idx_3mon,idx_3mon);

if (numObserved == 4) 
  var5Yr = nom5YrLoad * stateCovMat * nom5YrLoad' + H(idx_5yr,idx_5yr);
end

%
% regression coefficients and R^2s
%

threeMcoef = inv(varNext)*cov3Mnext';
explainVar3M = threeMcoef' * varNext * threeMcoef;
R2_3m = 1 - explainVar3M/var3M;

if (numObserved == 4) 
  fiveYRcoef1 = inv(varLong)*cov5YRlong';
  fiveYRcoef2 = inv(varLong3m)* cov5YRlong3m';
  explainVar5YR1 = fiveYRcoef1' * varLong * fiveYRcoef1;
  explainVar5YR2 = fiveYRcoef2' * varLong3m * fiveYRcoef2;
  R2_5YR_1 = 1 - explainVar5YR1/var5Yr;
  R2_5YR_2 = 1 - explainVar5YR2/var5Yr;
else
  fiveYRcoef1 = NaN;
  fiveYRcoef2 = NaN;
  R2_5YR_1 = NaN;
  R2_5YR_2 = NaN; 
end

% prepare output 

output.threeMcoef = threeMcoef;
output.fiveYRcoef1 = fiveYRcoef1; 
output.fiveYRcoef2 = fiveYRcoef2; 
output.R2_3m = R2_3m;
output.R2_5YR_1 = R2_5YR_1;
output.R2_5YR_2 = R2_5YR_2;

return
